@@ -11,25 +11,22 @@
 -- The Haddock API: A rudimentory, highly experimental API exposing some of
 -- the internals of Haddock. Don't expect it to be stable.
 -----------------------------------------------------------------------------
-
-
 module Documentation.Haddock (
 
   -- * Interface
   Interface(..),
   InstalledInterface(..),
   createInterfaces,
+  processModules,
 
   -- * Export items & declarations
   ExportItem(..),
-  DeclInfo,
   DocForDecl,
-  FnArgsDoc, 
- 
-  -- * Hyperlinking
+  FnArgsDoc,
+
+  -- * Cross-referencing
   LinkEnv,
   DocName(..),
-  docNameOcc,
 
   -- * Instances
   DocInstance,
@@ -37,17 +34,18 @@ module Documentation.Haddock (
 
   -- * Documentation comments
   Doc(..),
+  Example(..),
   DocMarkup(..),
   HaddockModInfo(..),
+  markup,
 
   -- * Interface files
-  -- | (.haddock files)
   InterfaceFile(..),
   readInterfaceFile,
   nameCacheFromGhc,
   freshNameCache,
   NameCacheAccessor,
-  
+
   -- * Flags and options
   Flag(..),
   DocOption(..)
@@ -59,3 +57,19 @@ import Haddock.InterfaceFile
 import Haddock.Interface
 import Haddock.Types
 import Haddock.Options
+import Haddock.Utils
+import Main
+
+
+-- | Create 'Interface' structures from a given list of Haddock command-line
+-- flags and file or module names (as accepted by 'haddock' executable).  Flags
+-- that control documentation generation or show help or version information
+-- are ignored.
+createInterfaces
+  :: [Flag]         -- ^ A list of command-line flags
+  -> [String]       -- ^ File or module names
+  -> IO [Interface] -- ^ Resulting list of interfaces
+createInterfaces flags modules = do
+  (_, ifaces, _) <- readPackagesAndProcessModules flags modules
+  return ifaces
+
